/* Copyright 2001, 2003 - Andrea Marchesini - bakunin
 *                        <bakunin@autistici.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <sdf.h>

/* Read the config file */
struct sdf_conf_t *
sdf_conf_init (void)
{

  struct passwd *pwd;
  struct sdf_conf_t *c;
  cfg_t *cfg;
  char *r;

  cfg_opt_t opts[] = {
    CFG_STR ("proxy", "", CFGF_NONE),
    CFG_INT ("port", 8080, CFGF_NONE),
    CFG_BOOL ("socks5", cfg_false, CFGF_NONE),
    CFG_END ()
  };

  if (!(pwd = getpwuid (getuid ())))
    return NULL;

  if (!(r = (char *) malloc ((strlen (pwd->pw_dir) + 8) * sizeof (char))))
    return NULL;

  sprintf (r, "%s/.sdfrc", pwd->pw_dir);

  D ("Reading %s file.\n", r);

  cfg = cfg_init (opts, 0);

  if (cfg_parse (cfg, r))
    {
      free (r);
      cfg_free (cfg);
      return NULL;
    }

  D ("Config OK.\n");

  if (!(c = (struct sdf_conf_t *) malloc (sizeof (struct sdf_conf_t))))
    {
      cfg_free (cfg);
      free (r);
      return NULL;
    }

  c->proxy = strdup (cfg_getstr (cfg, "proxy"));
  c->port = cfg_getint (cfg, "port");
  c->type = cfg_getbool (cfg, "socks5") ? CURLPROXY_SOCKS5 : CURLPROXY_HTTP;

  cfg_free (cfg);
  free (r);

  return c;
}

void
sdf_conf_cleanup (struct sdf_conf_t *c)
{

  if (!c)
    return;

  D ("Cleanup config.\n");
  free (c->proxy);
  free (c);
}
