/* Copyright 2001, 2003 - Andrea Marchesini - bakunin
 *                        <bakunin@autistici.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <sdf.h>

/* The wrapper of GNU function open(). If the first argoment is a remote file,
 * i use the libcurl to copy it inside the SDF directory and, after this i
 * use the real open on the local file. */
int
open (const char *path, int flags, ...)
{
  int (*r_open) (const char *, int, ...);
  va_list va;
  mode_t mode;
  CURL *curl;
  struct sdf_t sdf;
  struct sdf_conf_t *conf;
  int ret;

  va_start (va, flags);
  mode = va_arg (va, mode_t);
  va_end (va);

  if (!(r_open = dlsym (RTLD_NEXT, "open")))
    return -1;

  /* Remote file... */
  if (!strncmp (path, "http://", 7) || !strncmp (path, "ftp://", 6))
    {

      D ("Open start with a file distribuited.\n");

      sdf.fd = NULL;
      if (!(sdf.file = sdf_mk_name ()))
	return -1;

      conf = sdf_conf_init ();

      curl_global_init (CURL_GLOBAL_DEFAULT);

      if (!(curl = curl_easy_init ()))
	return -1;

      curl_easy_setopt (curl, CURLOPT_URL, path);
      curl_easy_setopt (curl, CURLOPT_WRITEDATA, &sdf);
      curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, sdf_fwrite);

      if (conf && conf->proxy)
	{
	  D ("Proxy: %s\n", conf->proxy);
	  curl_easy_setopt (curl, CURLOPT_PROXY, conf->proxy);

	  D ("Proxy: %d\n", conf->port);
	  curl_easy_setopt (curl, CURLOPT_PROXYPORT, conf->port);

	  D ("Proxy: %d\n", conf->type);
	  curl_easy_setopt (curl, CURLOPT_PROXYTYPE, conf->type);
	}

      if (curl_easy_perform (curl) != CURLE_OK)
	return -1;

      curl_easy_cleanup (curl);

      if (sdf.fd)
	fclose (sdf.fd);

      curl_global_cleanup ();

      if (chmod (sdf.file, S_IRUSR | S_ISVTX) < 0)
	return -1;

      D ("Get remote file and open the local copy.\n");
      ret = r_open (sdf.file, flags, mode);

      free (sdf.file);
      sdf_conf_cleanup (conf);

      return ret;
    }

  else
    return r_open (path, flags, mode);
}

int
open64 (const char *path, int flags, ...)
{
  va_list va;
  mode_t mode;

  va_start (va, flags);
  mode = va_arg (va, mode_t);
  va_end (va);

  return open (path, flags, mode);
}
